package org.especialistajee.jhd.filmoteca.dao;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.List;

import org.especialistajee.jhd.filmoteca.dao.DAOException;
import org.especialistajee.jhd.filmoteca.dao.MemoryPeliculaDAO;
import org.especialistajee.jhd.filmoteca.to.PeliculaTO;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;


public class MemoryPeliculaDAOTest {

	static TestableMemoryPeliculaDAO pdao;

	PeliculaTO pelicula1;
	PeliculaTO pelicula3;
	PeliculaTO peliculaNull;
	PeliculaTO peliculaEmpty;
	
	List<PeliculaTO> lista1;
	List<PeliculaTO> lista3;
	List<PeliculaTO> lista13;
	List<PeliculaTO> listaVacia;

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		pdao = new TestableMemoryPeliculaDAO();
	}

	@Before
	public void setUp() throws Exception {
		pelicula1 = new PeliculaTO();
		pelicula1.setId(1);
		pelicula1.setTitulo("El Resplandor");
		pelicula3 = new PeliculaTO();
		pelicula3.setId(3);
		pelicula3.setTitulo("Casablanca");
		peliculaNull = new PeliculaTO();
		peliculaNull.setId(4);
		peliculaEmpty = new PeliculaTO();
		peliculaEmpty.setId(5);
		peliculaEmpty.setTitulo("");

		lista1 = new ArrayList<PeliculaTO>();
		lista1.add(pelicula1);
		lista3 = new ArrayList<PeliculaTO>();
		lista3.add(pelicula3);
		lista13 = new ArrayList<PeliculaTO>();
		lista13.add(pelicula1);
		lista13.add(pelicula3);
		listaVacia = new ArrayList<PeliculaTO>();
	}

	@Test
	public void testAddPelicula1() {
		pdao.setAllPeliculas(listaVacia);
		try {
			pdao.addPelicula(pelicula1);
			assertEquals(lista1, pdao.getAllPeliculas());
		} catch (DAOException e) {
			fail("Capturada exception DAOException no esperada");
		}
	}

	@Test
	public void testAddPelicula2() {
		pdao.setAllPeliculas(lista1);
		try {
			pdao.addPelicula(pelicula3);
			assertEquals(lista13, pdao.getAllPeliculas());
		} catch (DAOException e) {
			fail("Capturada exception DAOException no esperada");
		}
	}

	@Test(expected=DAOException.class)
	public void testAddPelicula3() throws DAOException {
		pdao.setAllPeliculas(lista1);
		pdao.addPelicula(pelicula1);
	}

	@Test(expected=DAOException.class)
	public void testAddPelicula4() throws DAOException {
		pdao.setAllPeliculas(lista3);
		pdao.addPelicula(peliculaEmpty);
	}
	
	@Test(expected=DAOException.class)
	public void testAddPelicula5() throws DAOException {
		pdao.setAllPeliculas(lista3);
		pdao.addPelicula(peliculaNull);
	}
	
	@Test(expected=DAOException.class)
	public void testAddPelicula6() throws DAOException {
		pdao.setAllPeliculas(lista3);
		pdao.addPelicula(null);
	}
	
	@Test(expected=DAOException.class)
	public void testDelPelicula1() throws DAOException {
		pdao.setAllPeliculas(listaVacia);
		pdao.delPelicula(1);
	}

	@Test
	public void testDelPelicula2() {
		pdao.setAllPeliculas(lista1);
		try {
			pdao.delPelicula(1);
			assertEquals(listaVacia, pdao.getAllPeliculas());
		} catch (DAOException e) {
			fail("Capturada exception DAOException no esperada");
		}
	}
	
	@Test(expected=DAOException.class)
	public void testDelPelicula3() throws DAOException {
		pdao.setAllPeliculas(lista3);
		pdao.delPelicula(1);
	}
	
	@Test
	public void testDelPelicula4() {
		pdao.setAllPeliculas(lista13);
		try {
			pdao.delPelicula(1);
			assertEquals(lista3, pdao.getAllPeliculas());
		} catch (DAOException e) {
			fail("Capturada exception DAOException no esperada");
		}
	}

	@Test(expected=DAOException.class)
	public void testDelPelicula5() throws DAOException {
		pdao.setAllPeliculas(lista13);
		pdao.delPelicula(2);
	}

}

class TestableMemoryPeliculaDAO extends MemoryPeliculaDAO {
	public void setAllPeliculas(List<PeliculaTO> peliculas) {
		listaPeliculas = new ArrayList<PeliculaTO>(peliculas);
	}
}
