package org.especialistajee.jhd.tienda.bo;

import java.io.IOException;
import java.util.List;

import org.especialistajee.jhd.tienda.aeat.ProxyAeat;
import org.especialistajee.jhd.tienda.to.TipoEmpleado;
import org.especialistajee.jhd.tienda.to.TramoRetencion;


public class EmpleadoBR {
	
	public final static float SALARIO_BASE_VENDEDOR = 1000; 
	public final static float SALARIO_BASE_ENCARGADO = 1500; 
	
	public final static float SALARIO_HORA_EXTRA = 20; 	
	
	public final static float LIMITE_PRIMA_1 = 1500; 	
	public final static float SALARIO_PRIMA_1 = 200; 	
	public final static float LIMITE_PRIMA_2 = 1000; 	
	public final static float SALARIO_PRIMA_2 = 100; 	

	public float calculaSalarioBruto(TipoEmpleado tipo, float ventasMes, float horasExtra) {
		float salario = 0.0f;
		
		if(tipo == null) {
			throw new BRException("El tipo de empleado no puede ser nulo");
		} else if(ventasMes < 0) {
			throw new BRException("Las ventas mensuales no pueden ser negativas");
		} else if(horasExtra < 0) {
			throw new BRException("Las horas extra trabajadas no pueden ser negativas");
		}
		 
		if(tipo == TipoEmpleado.encargado) {
			salario += SALARIO_BASE_ENCARGADO;
		} else if(tipo == TipoEmpleado.vendedor) {
			salario += SALARIO_BASE_VENDEDOR;
		}
		
		if(ventasMes >= LIMITE_PRIMA_1) {
			salario += SALARIO_PRIMA_1;
		} else if(ventasMes >= LIMITE_PRIMA_2) {
			salario += SALARIO_PRIMA_2;
		}
		
		salario += horasExtra*SALARIO_HORA_EXTRA;
		
		return salario;
	}
	
	public float calculaSalarioNeto(float salarioBruto) {
		float retencion = 0.0f;
		
		if(salarioBruto<0) {
			throw new BRException("El salario bruto debe ser positivo");
		}
		
		ProxyAeat proxy = getProxyAeat();
		List<TramoRetencion> tramos;
		try {
			tramos = proxy.getTramosRetencion();
		} catch (IOException e) {
			throw new BRException("Error al conectar al servidor de la AEAT", e);
		} 
		
		for(TramoRetencion tr: tramos) {
			if(salarioBruto < tr.getLimiteSalario()) {
				retencion = tr.getRetencion();
				break;
			}
		}
		
		return salarioBruto * (1 - retencion); 
	}

	ProxyAeat getProxyAeat() {
		ProxyAeat proxy = new ProxyAeat();
		return proxy;
	}
}
