package org.especialistajee.jhd.tienda.bo;

import static junit.framework.Assert.*;

import org.especialistajee.jhd.tienda.bo.BOException;
import org.especialistajee.jhd.tienda.bo.EmpleadoBO;
import org.especialistajee.jhd.tienda.dao.DAOException;
import org.especialistajee.jhd.tienda.dao.IEmpleadoDAO;
import org.especialistajee.jhd.tienda.dao.MockEmpleadoDAO;
import org.especialistajee.jhd.tienda.to.EmpleadoTO;
import org.especialistajee.jhd.tienda.to.TipoEmpleado;
import org.junit.Before;
import org.junit.Test;


public class EmpleadoBOTest {

	TestableEmpleadoBO ebo;
	
	@Before
	public void setUp() {
		IEmpleadoDAO edao = new MockEmpleadoDAO(false);
		try {
			edao.addEmpleado(new EmpleadoTO(1,"12345678X", "Paco Garcia", TipoEmpleado.vendedor, 1250, 8));
			edao.addEmpleado(new EmpleadoTO(2,"65645342B", "Maria Gomez", TipoEmpleado.encargado, 1600, 2));
			edao.addEmpleado(new EmpleadoTO(3,"45452343F", "Manolo Gutierrez", TipoEmpleado.vendedor, 800, 0));
		} catch (DAOException e) {
			e.printStackTrace();
		}
		
		ebo = new TestableEmpleadoBO();
		ebo.setEmpleadoDAO(edao);
	}
	
	@Test
	public void testGetSalarioBruto() {
		try {
			float resultadoReal = ebo.getSalarioBruto(3);
			float resultadoEsperado = 1000.0f;
			
			assertEquals(resultadoEsperado, resultadoReal, 0.01);
		} catch (BOException e) {
			fail("Lanzada excepcion no esperada BOException");
		}
	}
}

class TestableEmpleadoBO extends EmpleadoBO {

	IEmpleadoDAO edao;
	
	public void setEmpleadoDAO(IEmpleadoDAO edao) {
		this.edao = edao;
	}
	
	@Override
	IEmpleadoDAO getEmpleadoDAO() {
		return edao;
	}
	
}