package org.especialistajee.jhd.tienda.bo;

import static org.junit.Assert.*;

import org.especialistajee.jhd.tienda.aeat.MockProxyAeat;
import org.especialistajee.jhd.tienda.aeat.ProxyAeat;
import org.especialistajee.jhd.tienda.bo.BRException;
import org.especialistajee.jhd.tienda.bo.EmpleadoBR;
import org.especialistajee.jhd.tienda.to.TipoEmpleado;
import org.junit.Before;
import org.junit.Test;


public class EmpleadoBRTest {

	TestableEmpleadoBR ebr;
	TestableEmpleadoBR ebrFail;
	
	@Before
	public void setUpClass() {
		ebr = new TestableEmpleadoBR();
		ebr.setProxyAeat(new MockProxyAeat(false));

		ebrFail = new TestableEmpleadoBR();
		ebrFail.setProxyAeat(new MockProxyAeat(true));
	}

	@Test
	public void testCalculaSalarioBruto1() {
		float resultadoReal;
		try {
			resultadoReal = ebr.calculaSalarioBruto(TipoEmpleado.vendedor, 2000.0f, 8.0f);
			float resultadoEsperado = 1360.0f;
			assertEquals(resultadoEsperado, resultadoReal, 0.01);
		} catch (BRException e) {
			fail("Lanzada excepcion no esperada BRException");
		}
	}
	@Test
	public void testCalculaSalarioBruto2() {
		float resultadoReal;
		try {
			resultadoReal = ebr.calculaSalarioBruto(TipoEmpleado.vendedor, 1500.0f, 3.0f);
			float resultadoEsperado = 1260.0f;
			assertEquals(resultadoEsperado, resultadoReal, 0.01);
		} catch (BRException e) {
			fail("Lanzada excepcion no esperada BRException");
		}
	}
	@Test
	public void testCalculaSalarioBruto3() {
		float resultadoReal;
		try {
			resultadoReal = ebr.calculaSalarioBruto(TipoEmpleado.vendedor, 1499.99f, 0.0f);
			float resultadoEsperado = 1100.0f;
			assertEquals(resultadoEsperado, resultadoReal, 0.01);
		} catch (BRException e) {
			fail("Lanzada excepcion no esperada BRException");
		}
	}
	@Test
	public void testCalculaSalarioBruto4() {
		float resultadoReal;
		try {
			resultadoReal = ebr.calculaSalarioBruto(TipoEmpleado.encargado, 1250.0f, 8.0f);
			float resultadoEsperado = 1760.0f;
			assertEquals(resultadoEsperado, resultadoReal, 0.01);
		} catch (BRException e) {
			fail("Lanzada excepcion no esperada BRException");
		}
	}
	@Test
	public void testCalculaSalarioBruto5() {
		float resultadoReal;
		try {
			resultadoReal = ebr.calculaSalarioBruto(TipoEmpleado.encargado, 1000.0f, 0.0f);
			float resultadoEsperado = 1600.0f;
			assertEquals(resultadoEsperado, resultadoReal, 0.01);
		} catch (BRException e) {
			fail("Lanzada excepcion no esperada BRException");
		}
	}
	@Test
	public void testCalculaSalarioBruto6() {
		float resultadoReal;
		try {
			resultadoReal = ebr.calculaSalarioBruto(TipoEmpleado.encargado, 999.99f, 3.0f);
			float resultadoEsperado = 1560.0f;
			assertEquals(resultadoEsperado, resultadoReal, 0.01);
		} catch (BRException e) {
			fail("Lanzada excepcion no esperada BRException");
		}
	}
	@Test
	public void testCalculaSalarioBruto7() {
		float resultadoReal;
		try {
			resultadoReal = ebr.calculaSalarioBruto(TipoEmpleado.encargado, 500.0f, 0.0f);
			float resultadoEsperado = 1500.0f;
			assertEquals(resultadoEsperado, resultadoReal, 0.01);
		} catch (BRException e) {
			fail("Lanzada excepcion no esperada BRException");
		}
	}
	@Test
	public void testCalculaSalarioBruto8() {
		float resultadoReal;
		try {
			resultadoReal = ebr.calculaSalarioBruto(TipoEmpleado.encargado, 0.0f, 8.0f);
			float resultadoEsperado = 1660.0f;
			assertEquals(resultadoEsperado, resultadoReal, 0.01);
		} catch (BRException e) {
			fail("Lanzada excepcion no esperada BRException");
		}
	}

	@Test(expected=BRException.class)
	public void testCalculaSalarioBruto9() {
		ebr.calculaSalarioBruto(TipoEmpleado.vendedor, -1.0f, 8.0f);
	}
	@Test(expected=BRException.class)
	public void testCalculaSalarioBruto10() {
		ebr.calculaSalarioBruto(TipoEmpleado.vendedor, 1500.0f, -1.0f);
	}
	@Test(expected=BRException.class)
	public void testCalculaSalarioBruto11() {
		ebr.calculaSalarioBruto(null, 1500.0f, 8.0f);
	}

	@Test
	public void testCalculaSalarioNeto1() {
		float resultadoReal = ebr.calculaSalarioNeto(2000.0f);
		float resultadoEsperado = 1640.0f;
		assertEquals(resultadoEsperado, resultadoReal, 0.01);
	}

	@Test
	public void testCalculaSalarioNeto2() {
		float resultadoReal = ebr.calculaSalarioNeto(1500.0f);
		float resultadoEsperado = 1230.0f;
		assertEquals(resultadoEsperado, resultadoReal, 0.01);
	}
	
	@Test
	public void testCalculaSalarioNeto3() {
		float resultadoReal = ebr.calculaSalarioNeto(1499.99f);
		float resultadoEsperado = 1259.9916f;
		assertEquals(resultadoEsperado, resultadoReal, 0.01);
	}
	
	@Test
	public void testCalculaSalarioNeto4() {
		float resultadoReal = ebr.calculaSalarioNeto(1250.0f);
		float resultadoEsperado = 1050.0f;
		assertEquals(resultadoEsperado, resultadoReal, 0.01);
	}

	@Test
	public void testCalculaSalarioNeto5() {
		float resultadoReal = ebr.calculaSalarioNeto(1000.0f);
		float resultadoEsperado = 840.0f;
		assertEquals(resultadoEsperado, resultadoReal, 0.01);
	}

	@Test
	public void testCalculaSalarioNeto6() {
		float resultadoReal = ebr.calculaSalarioNeto(999.99f);
		float resultadoEsperado = 999.99f;
		assertEquals(resultadoEsperado, resultadoReal, 0.01);
	}

	@Test
	public void testCalculaSalarioNeto7() {
		float resultadoReal = ebr.calculaSalarioNeto(500.0f);
		float resultadoEsperado = 500.0f;
		assertEquals(resultadoEsperado, resultadoReal, 0.01);
	}
	
	@Test
	public void testCalculaSalarioNeto8() {
		float resultadoReal = ebr.calculaSalarioNeto(0.0f);
		float resultadoEsperado = 0.0f;
		assertEquals(resultadoEsperado, resultadoReal, 0.01);
	}

	@Test(expected=BRException.class)
	public void testCalculaSalarioNeto9() {
		ebr.calculaSalarioNeto(-1.0f);
	}

	@Test(expected=BRException.class)
	public void testCalculaSalarioNeto10() {
		ebrFail.calculaSalarioNeto(1000.0f);
	}
}

class TestableEmpleadoBR extends EmpleadoBR {

	ProxyAeat proxy;
	
	public void setProxyAeat(ProxyAeat proxy) {
		this.proxy = proxy;
	}
	
	@Override
	ProxyAeat getProxyAeat() {
		return proxy;
	}
}
